@@ -9,6 +9,8 @@
 #include <sys/socket.h>
 #include <sys/errno.h>
 #include <netdb.h>
+#define SYSLOG_NAMES
+#include <sys/syslog.h>
 #include <ifaddrs.h>
 #include <arpa/inet.h>
 
@@ -85,6 +87,31 @@ __attribute__ ((constructor)) static void cmanpre_comp_register(void) {
 	lcr_component_register(&cmanpre_comp_ver0);
 }
 
+static char *facility_name_get (unsigned int facility)
+{
+	unsigned int i;
+
+	for (i = 0; facilitynames[i].c_name != NULL; i++) {
+		if (facility == facilitynames[i].c_val) {
+			return (facilitynames[i].c_name);
+		}
+	}
+	return (NULL);
+}
+
+static char *priority_name_get (unsigned int priority)
+{
+	unsigned int i;
+
+	for (i = 0; prioritynames[i].c_name != NULL; i++) {
+		if (priority == prioritynames[i].c_val) {
+			return (prioritynames[i].c_name);
+		}
+	}
+	return (NULL);
+}
+
+
 #define LOCALHOST_IPV4 "127.0.0.1"
 #define LOCALHOST_IPV6 "::1"
 
@@ -487,9 +514,7 @@ static int get_nodename(struct objdb_iface_ver0 *objdb)
 
 			strcpy(nodename, nodename_env);
 
-			if (objdb->object_find(object_handle,
-					       nodename, strlen(nodename),
-					       &node_object_handle) != 0) {
+			if (!(node_object_handle = nodelist_byname(objdb, cluster_parent_handle, nodename))) {
 				sprintf(error_reason, "Overridden node name %s is not in CCS", nodename);
 				write_cman_pipe("Overridden node name is not in CCS");
 				error = -1;
@@ -711,7 +736,7 @@ static void add_cman_overrides(struct objdb_iface_ver0 *objdb)
 		char *logstr;
 		char *logfacility;
 
-		logfacility = logsys_facility_name_get(SYSLOGFACILITY);
+		logfacility = facility_name_get(SYSLOGFACILITY);
 
 		logger_object_handle = find_cman_logger(objdb, object_handle);
 
@@ -741,7 +766,7 @@ static void add_cman_overrides(struct objdb_iface_ver0 *objdb)
 		}
 		else {
 			char *loglevel;
-			loglevel = logsys_priority_name_get(SYSLOGLEVEL);
+			loglevel = priority_name_get(SYSLOGLEVEL);
 			objdb->object_key_create(logger_object_handle, "syslog_level", strlen("syslog_level"),
 						 loglevel, strlen(loglevel)+1);
 		}
