@@ -1,13 +1,15 @@
 #ifndef _DBG_H
 #define _DBG_H
 
+#include <openais/service/logsys.h>
+
 inline void dset(int);
 inline int dget(void);
 
 #define dbg_printf(level, fmt, args...) \
 do { \
 	if (dget()>=level) \
-		printf(fmt, ##args); \
+		log_printf(LOG_DEBUG, fmt, ##args); \
 } while(0)
 
 #endif
