@@ -1,21 +1,3 @@
-/*
-  Copyright Red Hat, Inc. 2006
-
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2, or (at your option) any
-  later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING.  If not, write to the
-  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
-  MA 02139, USA.
-*/
 /*
  * @file fence_xvmd.c: Implementation of server daemon for Xen virtual
  * machine fencing.  This uses SA AIS CKPT b.1.0 checkpointing API to 
@@ -55,6 +37,13 @@
 #include "mcast.h"
 #include "debug.h"
 
+LOGSYS_DECLARE_SYSTEM (NULL,
+        LOG_MODE_OUTPUT_STDERR | LOG_MODE_OUTPUT_SYSLOG_THREADED |
+	LOG_MODE_OUTPUT_FILE,
+        LOGDIR "/fence_xvm.log",
+        SYSLOGFACILITY);
+
+LOGSYS_DECLARE_SUBSYS ("XVM", SYSLOGLEVEL);
 
 int
 tcp_wait_connect(int lfd, int retry_tenths)
@@ -98,14 +87,14 @@ tcp_exchange(int fd, fence_auth_type_t auth, void *key,
 	dbg_printf(3, "Issuing TCP challenge\n");
 	if (tcp_challenge(fd, auth, key, key_len, timeout) <= 0) {
 		/* Challenge failed */
-		printf("Invalid response to challenge\n");
+		log_printf(LOG_ERR, "Invalid response to challenge\n");
 		return 0;
 	}
 
 	/* Now they'll send us one, so we need to respond here */
 	dbg_printf(3, "Responding to TCP challenge\n");
 	if (tcp_response(fd, auth, key, key_len, timeout) <= 0) {
-		printf("Invalid response to challenge\n");
+		log_printf(LOG_ERR, "Invalid response to challenge\n");
 		return 0;
 	}
 
@@ -127,9 +116,9 @@ tcp_exchange(int fd, fence_auth_type_t auth, void *key,
 
 	close(fd);
 	if (ret == 0)
-		printf("Remote: Operation was successful\n");
+		log_printf(LOG_INFO, "Remote: Operation was successful\n");
 	else
-		printf("Remote: Operation failed\n");
+		log_printf(LOG_INFO, "Remote: Operation failed\n");
 	return ret;
 }
 
@@ -228,8 +217,9 @@ fence_xen_domain(fence_xvm_args_t *args)
 	if (args->auth != AUTH_NONE || args->hash != HASH_NONE) {
 		key_len = read_key_file(args->key_file, key, sizeof(key));
 		if (key_len < 0) {
-			printf("Could not read %s; trying without "
-			       "authentication\n", args->key_file);
+			log_printf(LOG_INFO,
+				   "Could not read %s; trying without "
+			           "authentication\n", args->key_file);
 			args->auth = AUTH_NONE;
 			args->hash = HASH_NONE;
 		}
@@ -237,7 +227,7 @@ fence_xen_domain(fence_xvm_args_t *args)
 
 	/* Do the real work */
 	if (ip_build_list(&ipl) < 0) {
-		printf("Error building IP address list\n");
+		log_printf(LOG_ERR, "Error building IP address list\n");
 		return 1;
 	}
 
@@ -259,12 +249,15 @@ fence_xen_domain(fence_xvm_args_t *args)
 	}
 
 	if (lfd < 0) {
-		printf("Failed to listen: %s\n", strerror(errno));
+		log_printf(LOG_ERR, "Failed to listen: %s\n", strerror(errno));
 		return 1;
 	}
 
 	attempts = args->timeout * 10 / args->retr_time;
 
+	log_printf(LOG_INFO, "Sending fence request for %s\n", 
+		   args->domain);
+
 	do {
 		if (send_multicast_packets(&ipl, args, key, key_len)) {
 			return -1;
@@ -292,10 +285,11 @@ fence_xen_domain(fence_xvm_args_t *args)
 
 	if (fd < 0) {
 		if (attempts <= 0) {
-			printf("Timed out waiting for response\n");
+			log_printf(LOG_ERR,
+				   "Timed out waiting for response\n");
 			return 1;
 		}
-		printf("Fencing failed: %s\n", strerror(errno));
+		log_printf(LOG_ERR, "Fencing failed: %s\n", strerror(errno));
 		return -1;
 	}
 
@@ -324,6 +318,7 @@ main(int argc, char **argv)
 	char *my_options = "di:a:p:T:r:C:c:k:H:uo:t:?hV";
 
 	args_init(&args);
+
 	if (argc == 1) {
 		args_get_stdin(my_options, &args);
 	} else {
@@ -332,6 +327,14 @@ main(int argc, char **argv)
 
 	if (args.flags & F_HELP) {
 		args_usage(argv[0], my_options, 0);
+
+                printf("With no command line argument, arguments are "
+                       "read from standard input.\n");
+                printf("Arguments read from standard input take "
+                       "the form of:\n\n");
+                printf("    arg1=value1\n");
+                printf("    arg2=value2\n\n");
+
 		args_usage(argv[0], my_options, 1);
 		exit(0);
 	}
@@ -347,12 +350,14 @@ main(int argc, char **argv)
 	args_finalize(&args);
 	dset(args.debug);
 	
-	if (args.debug > 0) 
+	if (args.debug > 0) {
+                logsys_config_priority_set (LOG_LEVEL_DEBUG);
 		args_print(&args);
+	}
 
 	/* Additional validation here */
 	if (!args.domain) {
-		printf("No domain specified!\n");
+		log_printf(LOG_ERR, "No domain specified!\n");
 		args.flags |= F_ERR;
 	}
 
@@ -364,7 +369,7 @@ main(int argc, char **argv)
 	/* Initialize NSS; required to do hashing, as silly as that
 	   sounds... */
 	if (NSS_NoDB_Init(NULL) != SECSuccess) {
-		printf("Could not initialize NSS\n");
+		log_printf(LOG_ERR, "Could not initialize NSS\n");
 		return 1;
 	}
 
