@@ -59,7 +59,6 @@
 #define GROUP_LIBCPG            3
 
 extern int daemon_debug_opt;
-extern int daemon_debug_logsys;
 extern int daemon_quit;
 extern struct list_head domains;
 extern int cman_quorate;
@@ -79,7 +78,7 @@ do { \
 	daemon_dump_save(); \
 	if (daemon_debug_opt) \
 		fprintf(stderr, "%s", daemon_debug_buf); \
-	if (daemon_debug_logsys) \
+	if (cfgd_debug_logsys) \
 		log_printf(LOG_DEBUG, "%s", daemon_debug_buf); \
 } while (0)
 
@@ -95,34 +94,6 @@ do { \
 	log_printf(lvl, fmt, ##args); \
 } while (0)
 
-/* config option defaults */
-
-#define DEFAULT_GROUPD_COMPAT	1
-#define DEFAULT_CLEAN_START	0
-#define DEFAULT_POST_JOIN_DELAY	6
-#define DEFAULT_POST_FAIL_DELAY	0
-#define DEFAULT_OVERRIDE_TIME   3
-#define DEFAULT_OVERRIDE_PATH	"/var/run/cluster/fenced_override"
-
-struct commandline
-{
-	int groupd_compat;
-	int clean_start;
-	int post_join_delay;
-	int post_fail_delay;
-	int override_time;
-	char *override_path;
-
-	int8_t groupd_compat_opt;
-	int8_t clean_start_opt;
-	int8_t post_join_delay_opt;
-	int8_t post_fail_delay_opt;
-	int8_t override_time_opt;
-	int8_t override_path_opt;
-};
-
-extern struct commandline comline;
-
 #define FD_MSG_START		1
 #define FD_MSG_VICTIM_DONE	2
 #define FD_MSG_COMPLETE		3
@@ -195,6 +166,7 @@ struct fd {
 	int			cpg_client;
 	int			cpg_fd;
 	uint32_t		change_seq;
+	uint32_t		started_count;
 	struct change		*started_change;
 	struct list_head	changes;
 	struct list_head	node_history;
@@ -231,10 +203,11 @@ int read_ccs(struct fd *fd);
 /* cpg.c */
 
 void free_cg(struct change *cg);
-void node_history_fence(struct fd *fd, int nodeid, int master, int how);
+void node_history_fence(struct fd *fd, int victim, int master, int how,
+			uint64_t mastertime);
 void send_external(struct fd *fd, int victim);
 int is_fenced_external(struct fd *fd, int nodeid);
-void send_victim_done(struct fd *fd, int victim, int how);
+void send_victim_done(struct fd *fd, int victim);
 void process_fd_changes(void);
 int fd_join(struct fd *fd);
 int fd_leave(struct fd *fd);
@@ -254,6 +227,7 @@ int set_node_info_group(struct fd *fd, int nodeid, struct fenced_node *node);
 int set_domain_info_group(struct fd *fd, struct fenced_domain *domain);
 int set_domain_nodes_group(struct fd *fd, int option, int *node_count,
 			   struct fenced_node **nodes);
+void set_group_mode(void);
 
 /* main.c */
 
@@ -288,7 +262,7 @@ void fence_victims(struct fd *fd);
 /* logging.c */
 
 void init_logging(void);
-void setup_logging(int *prog_debug);
+void setup_logging();
 void close_logging(void);
 
 #endif				/*  __FD_DOT_H__  */
