@@ -1,5 +1,6 @@
 #include "fd.h"
-#include "pthread.h"
+#include "config.h"
+#include <pthread.h>
 #include "copyright.cf"
 
 #define LOCKFILE_NAME		"/var/run/fenced.pid"
@@ -315,11 +316,11 @@ static void query_dump_debug(int f)
 	do_write(f, dump_buf, len);
 }
 
-static void query_node_info(int f, int nodeid)
+static void query_node_info(int f, int data_nodeid)
 {
 	struct fd *fd;
 	struct fenced_node node;
-	int rv;
+	int nodeid, rv;
 
 	fd = find_fd("default");
 	if (!fd) {
@@ -327,6 +328,11 @@ static void query_node_info(int f, int nodeid)
 		goto out;
 	}
 
+	if (data_nodeid == FENCED_NODEID_US)
+		nodeid = our_nodeid;
+	else
+		nodeid = data_nodeid;
+
 	if (group_mode == GROUP_LIBGROUP)
 		rv = set_node_info_group(fd, nodeid, &node);
 	else
@@ -623,23 +629,21 @@ static void loop(void)
 	if (rv < 0)
 		goto out;
 
-	setup_logging(&daemon_debug_logsys);
+	setup_logging();
 
 	group_mode = GROUP_LIBCPG;
 
-	if (comline.groupd_compat) {
+	if (cfgd_groupd_compat) {
 		rv = setup_groupd();
 		if (rv < 0)
 			goto out;
 		client_add(rv, process_groupd, cluster_dead);
 
 		group_mode = GROUP_LIBGROUP;
-
-		if (comline.groupd_compat == 2) {
-			/* set_group_mode(); */
-			group_mode = GROUP_LIBGROUP;
-		}
+		if (cfgd_groupd_compat == 2)
+			set_group_mode();
 	}
+	log_debug("group_mode %d compat %d", group_mode, cfgd_groupd_compat);
 
 	if (group_mode == GROUP_LIBCPG) {
 		/*
@@ -683,7 +687,7 @@ static void loop(void)
 			break;
 	}
  out:
-	if (comline.groupd_compat)
+	if (cfgd_groupd_compat)
 		close_groupd();
 	close_logging();
 	close_ccs();
@@ -744,11 +748,13 @@ static void print_usage(void)
 	printf("Options:\n");
 	printf("\n");
 	printf("  -D           Enable debugging code and don't fork\n");
-	printf("  -g <num>     groupd compatibility, 0 off, 1 on\n");
-	printf("               on: use libgroup, compat with cluster2/stable2/rhel5\n");
-	printf("               off: use libcpg, no backward compatability\n");
-	printf("               Default is %d\n", DEFAULT_GROUPD_COMPAT);
-	printf("  -c	       All nodes are in a clean state to start\n");
+	printf("  -L <num>     Enable (1) or disable (0) debugging to logsys (default %d)\n", DEFAULT_DEBUG_LOGSYS);
+	printf("  -g <num>     groupd compatibility mode, 0 off, 1 on, 2 detect (default %d)\n", DEFAULT_GROUPD_COMPAT);
+	printf("               0: use libcpg, no backward compat, best performance\n");
+	printf("               1: use libgroup for compat with cluster2/rhel5\n");
+	printf("               2: use groupd to detect old, or mode 1, nodes that\n"
+	       "               require compat, use libcpg if none found\n");
+	printf("  -c           All nodes are in a clean state to start\n");
 	printf("  -j <secs>    Post-join fencing delay (default %d)\n", DEFAULT_POST_JOIN_DELAY);
 	printf("  -f <secs>    Post-fail fencing delay (default %d)\n", DEFAULT_POST_FAIL_DELAY);
 	printf("  -R <secs>    Override time (default %d)\n", DEFAULT_OVERRIDE_TIME);
@@ -762,7 +768,7 @@ static void print_usage(void)
 	printf("\n");
 }
 
-#define OPTION_STRING	"g:cj:f:Dn:O:T:hVS"
+#define OPTION_STRING	"L:g:cj:f:Dn:O:T:hVS"
 
 static void read_arguments(int argc, char **argv)
 {
@@ -778,38 +784,41 @@ static void read_arguments(int argc, char **argv)
 			daemon_debug_opt = 1;
 			break;
 
+		case 'L':
+			optd_debug_logsys = 1;
+			cfgd_debug_logsys = atoi(optarg);
+			break;
+
 		case 'g':
-			comline.groupd_compat = atoi(optarg);
-			comline.groupd_compat_opt = 1;
+			optd_groupd_compat = 1;
+			cfgd_groupd_compat = atoi(optarg);
 			break;
 
 		case 'c':
-			comline.clean_start = 1;
-			comline.clean_start_opt = 1;
+			optd_clean_start = 1;
+			cfgd_clean_start = 1;
 			break;
 
 		case 'j':
-			comline.post_join_delay = atoi(optarg);
-			comline.post_join_delay_opt = 1;
+			optd_post_join_delay = 1;
+			cfgd_post_join_delay = atoi(optarg);
 			break;
 
 		case 'f':
-			comline.post_fail_delay = atoi(optarg);
-			comline.post_fail_delay_opt = 1;
+			optd_post_fail_delay = 1;
+			cfgd_post_fail_delay = atoi(optarg);
 			break;
 
 		case 'R':
-			comline.override_time = atoi(optarg);
-			if (comline.override_time < 3)
-				comline.override_time = 3;
-			comline.override_time_opt = 1;
+			optd_override_time = 1;
+			cfgd_override_time = atoi(optarg);
+			if (cfgd_override_time < 3)
+				cfgd_override_time = 3;
 			break;
 
 		case 'O':
-			if (comline.override_path)
-				free(comline.override_path);
-			comline.override_path = strdup(optarg);
-			comline.override_path_opt = 1;
+			optd_override_path = 1;
+			cfgd_override_path = strdup(optarg);
 			break;
 
 		case 'h':
@@ -839,6 +848,11 @@ static void read_arguments(int argc, char **argv)
 			exit(EXIT_FAILURE);
 		};
 	}
+
+	if (!optd_debug_logsys && getenv("FENCED_DEBUG")) {
+		optd_debug_logsys = 1;
+		cfgd_debug_logsys = atoi(getenv("FENCED_DEBUG"));
+	}
 }
 
 static void set_oom_adj(int val)
@@ -857,14 +871,6 @@ int main(int argc, char **argv)
 {
 	INIT_LIST_HEAD(&domains);
 
-	memset(&comline, 0, sizeof(comline));
-	comline.groupd_compat = DEFAULT_GROUPD_COMPAT;
-	comline.clean_start = DEFAULT_CLEAN_START;
-	comline.post_join_delay = DEFAULT_POST_JOIN_DELAY;
-	comline.post_fail_delay = DEFAULT_POST_FAIL_DELAY;
-	comline.override_time = DEFAULT_OVERRIDE_TIME;
-	comline.override_path = strdup(DEFAULT_OVERRIDE_PATH);
-
 	init_logging();
 
 	read_arguments(argc, argv);
@@ -904,7 +910,6 @@ void daemon_dump_save(void)
 }
 
 int daemon_debug_opt;
-int daemon_debug_logsys;
 int daemon_quit;
 struct list_head domains;
 int cman_quorate;
@@ -915,5 +920,4 @@ char dump_buf[FENCED_DUMP_SIZE];
 int dump_point;
 int dump_wrap;
 int group_mode;
-struct commandline comline;
 
