@@ -1,4 +1,5 @@
 #include "dlm_daemon.h"
+#include "config.h"
 #include "libgroup.h"
 
 #define DO_STOP 1
@@ -259,3 +260,31 @@ int set_lockspace_nodes_group(struct lockspace *ls, int option, int *node_count,
 	return 0;
 }
 
+void set_group_mode(void)
+{
+	int i = 0, rv, version;
+
+	while (1) {
+		rv = group_get_version(&version);
+		if (rv) {
+			log_error("group_get_version error %d", rv);
+			break;
+		} else if (version == GROUP_LIBGROUP) {
+			group_mode = GROUP_LIBGROUP;
+			break;
+		} else if (version == GROUP_LIBCPG) {
+			group_mode = GROUP_LIBCPG;
+			break;
+		} else if (version != -EAGAIN) {
+			log_error("group_get_version result %d", version);
+			break;
+		}
+
+		if (i++ > 10) {
+			log_error("no version from groupd");
+			break;
+		}
+		sleep(1);
+	}
+}
+
