@@ -1,6 +1,8 @@
 #include "dlm_daemon.h"
 #include "config.h"
 #include <pthread.h>
+#include "copyright.cf"
+
 #include <linux/dlmconstants.h>
 #include <linux/netlink.h>
 #include <linux/genetlink.h>
@@ -8,14 +10,11 @@
 
 #define LOCKFILE_NAME	"/var/run/dlm_controld.pid"
 #define CLIENT_NALLOC	32
-#define GROUP_LIBGROUP	2
-#define GROUP_LIBCPG	3
 
 static int client_maxi;
 static int client_size = 0;
 static struct client *client = NULL;
 static struct pollfd *pollfd = NULL;
-static int group_mode;
 static pthread_t query_thread;
 static pthread_mutex_t query_mutex;
 
@@ -820,6 +819,8 @@ static void loop(void)
 	if (rv < 0)
 		goto out;
 
+	setup_logging();
+
 	rv = setup_configfs();
 	if (rv < 0)
 		goto out;
@@ -838,32 +839,10 @@ static void loop(void)
 		client_add(rv, process_groupd, cluster_dead);
 
 		group_mode = GROUP_LIBGROUP;
-
-		if (cfgd_groupd_compat == 2) {
-			/* cfgd_groupd_compat of 2 uses new groupd feature that
-			   figures out whether all other groupd's in the cluster
-			   are in LIGCPG mode, and if they are group_mode is
-			   changed to LIBCPG.  If any groupd in the cluster
-			   is from cluster2/stable2/rhel5, or any groupd is
-			   in LIBGROUP mode, then group_mode remains LIBGROUP.
-
-			   set_group_mode() figures this out by joining the
-			   groupd cpg, sending a new "mode" message, and
-			   waiting to see if it gets a mode reply from
-			   all other groupd's.  If it does, and all modes
-			   are LIGCPG, then we set groupd_mode to LIBCPG.
-			   Any previous generation groupd's won't recognize
-			   the new mode message and won't reply; their lack
-			   of reply (or seeing an old-style message from them)
-			   indicates they are a cluster2 version.
-
-			   Not yet implemented.  In the future, it may set
-			   group_mode to GROUP_LIBCPG. */
-
-			/* set_group_mode(); */
-			group_mode = GROUP_LIBGROUP;
-		}
+		if (cfgd_groupd_compat == 2)
+			set_group_mode();
 	}
+	log_debug("group_mode %d compat %d", group_mode, cfgd_groupd_compat);
 
 	if (group_mode == GROUP_LIBCPG) {
 		rv = setup_cpg();
@@ -939,6 +918,7 @@ static void loop(void)
 	if (cfgd_groupd_compat)
 		close_groupd();
 	clear_configfs();
+	close_logging();
 	close_ccs();
 	close_cman();
 
@@ -997,10 +977,13 @@ static void print_usage(void)
 	printf("Options:\n");
 	printf("\n");
 	printf("  -D		Enable daemon debugging and don't fork\n");
+	printf("  -L <num>	Enable (1) or disable (0) debugging to logsys (default %d)\n", DEFAULT_DEBUG_LOGSYS);
 	printf("  -K		Enable kernel dlm debugging messages\n");
-	printf("  -g <num>	groupd compatibility, 0 off, 1 on\n");
-	printf("		on: use libgroup, compat with cluster2/stable2/rhel5\n");
-	printf("		off: use libcpg, no backward compatability\n");
+	printf("  -g <num>	groupd compatibility mode, 0 off, 1 on, 2 detect\n");
+	printf("		0: use libcpg, no backward compat, best performance\n");
+	printf("		1: use libgroup for compat with cluster2/rhel5\n");
+	printf("		2: use groupd to detect old, or mode 1, nodes that\n"
+	       "		require compat, use libcpg if none found\n");
 	printf("		Default is %d\n", DEFAULT_GROUPD_COMPAT);
 	printf("  -f <num>	Enable (1) or disable (0) fencing recovery dependency\n");
 	printf("		Default is %d\n", DEFAULT_ENABLE_FENCING);
@@ -1025,7 +1008,7 @@ static void print_usage(void)
 	printf("  -V		Print program version information, then exit\n");
 }
 
-#define OPTION_STRING			"DKg:f:q:d:p:Pl:o:t:c:a:hV"
+#define OPTION_STRING "L:DKg:f:q:d:p:Pl:o:t:c:a:hV"
 
 static void read_arguments(int argc, char **argv)
 {
@@ -1045,6 +1028,11 @@ static void read_arguments(int argc, char **argv)
 			daemon_debug_opt = 1;
 			break;
 
+		case 'L':
+			optd_debug_logsys = 1;
+			cfgd_debug_logsys = atoi(optarg);
+			break;
+
 		case 'g':
 			optd_groupd_compat = 1;
 			cfgd_groupd_compat = atoi(optarg);
@@ -1113,7 +1101,7 @@ static void read_arguments(int argc, char **argv)
 		case 'V':
 			printf("dlm_controld %s (built %s %s)\n",
 				RELEASE_VERSION, __DATE__, __TIME__);
-			/* printf("%s\n", REDHAT_COPYRIGHT); */
+			printf("%s\n", REDHAT_COPYRIGHT);
 			exit(EXIT_SUCCESS);
 			break;
 
@@ -1133,6 +1121,11 @@ static void read_arguments(int argc, char **argv)
 			break;
 		};
 	}
+
+	if (!optd_debug_logsys && getenv("DLM_CONTROLD_DEBUG")) {
+		optd_debug_logsys = 1;
+		cfgd_debug_logsys = atoi(getenv("DLM_CONTROLD_DEBUG"));
+	}
 }
 
 static void set_oom_adj(int val)
@@ -1169,6 +1162,8 @@ int main(int argc, char **argv)
 {
 	INIT_LIST_HEAD(&lockspaces);
 
+	init_logging();
+
 	read_arguments(argc, argv);
 
 	lockfile();
@@ -1179,7 +1174,6 @@ int main(int argc, char **argv)
 			exit(EXIT_FAILURE);
 		}
 	}
-	openlog("dlm_controld", LOG_PID, LOG_DAEMON);
 	signal(SIGTERM, sigterm_handler);
 
 	set_scheduler();
@@ -1223,4 +1217,5 @@ int dump_point;
 int dump_wrap;
 char plock_dump_buf[DLMC_DUMP_SIZE];
 int plock_dump_len;
+int group_mode;
 
