@@ -1,26 +1,7 @@
-/*
-  Copyright Red Hat, Inc. 2002
-  Copyright Mission Critical Linux, 2000
-
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2, or (at your option) any
-  later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING.  If not, write to the
-  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
-  MA 02139, USA.
-*/
 /** @file
  * Library routines for communicating with the logging daemon.
  *
- *  $Id: clulog.c,v 1.7 2007/04/30 11:22:23 fabbione Exp $
+ *  $Id$
  *
  *  Author: Jeff Moyer <moyer@missioncriticallinux.com>
  */
@@ -49,7 +30,7 @@
 #include <string.h>
 
 
-static const char *version __attribute__ ((unused)) = "$Revision: 1.7 $";
+static const char *version __attribute__ ((unused)) = "$Revision$";
 
 #ifdef DEBUG
 #include <assert.h>
@@ -282,7 +263,7 @@ do_clulog(int        severity,
 		fprintf(stdout, "%s", printmsg);
 	}
 
-	syslog(severity, logmsg);
+	syslog(severity, "%s", logmsg);
 
 	pthread_mutex_unlock(&log_mutex);
 
